@@ -158,6 +158,10 @@ uint32_t DataTree::_numLeaves(const DataNode &node) const {
 }
 
 void DataTree::traverseLeaves(uint32_t beginIndex, uint32_t endIndex, function<void (DataLeafNode*, uint32_t)> func) {
+  if (endIndex <= beginIndex) {
+      return;
+  }
+  
   //TODO Can we traverse in parallel?
   unique_lock<shared_mutex> lock(_mutex); //TODO Only lock when resizing. Otherwise parallel read/write to a blob is not possible!
   ASSERT(beginIndex <= endIndex, "Invalid parameters");
