@@ -4,7 +4,7 @@
 
 #include "HttpClient.h"
 #include "../macros.h"
-#include <curl/curl.h>
+#include "CurlInitializerRAII.h"
 
 namespace cpputils {
 
@@ -17,6 +17,7 @@ namespace cpputils {
         boost::optional <std::string> get(const std::string &url, boost::optional<long> timeoutMsec = boost::none) override;
 
     private:
+        CurlInitializerRAII curlInitializer;
         CURL *curl;
 
         static size_t write_data(void *ptr, size_t size, size_t nmemb, std::ostringstream *stream);
